@@ -10,7 +10,7 @@ style = color_style()
 
 class AppConfig(DjangoAppConfig):
     name = "edc_metadata"
-    verbose_name = "Edc Metadata"
+    verbose_name = "Data collection status"
     metadata_rules_enabled = True
     has_exportable_data = True
     include_in_administration_section = True
