@@ -1,3 +1,7 @@
+from __future__ import annotations
+
+from typing import Type
+
 from .metadata_getter import MetadataGetter, MetadataValidator
 
 
@@ -8,7 +12,6 @@ class RequisitionMetadataValidator(MetadataValidator):
 
 
 class RequisitionMetadataGetter(MetadataGetter):
+    metadata_model: str = "edc_metadata.requisitionmetadata"
 
-    metadata_model = "edc_metadata.requisitionmetadata"
-
-    metadata_validator_cls = RequisitionMetadataValidator
+    metadata_validator_cls: Type[RequisitionMetadataValidator] = RequisitionMetadataValidator
