@@ -1,4 +1,6 @@
-from typing import Any, Optional
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any
 from warnings import warn
 
 from django.apps import apps as django_apps
@@ -8,6 +10,16 @@ from django.core.exceptions import ObjectDoesNotExist
 from ..constants import DO_NOTHING
 from .predicate import NoValueError
 
+if TYPE_CHECKING:
+    from edc_registration.models import RegisteredSubject
+    from edc_visit_tracking.model_mixins import VisitModelMixin as Base
+
+    from ..model_mixins.creates import CreatesMetadataModelMixin
+    from .logic import Logic
+
+    class RelatedVisitModel(CreatesMetadataModelMixin, Base):
+        pass
+
 
 class RuleEvaluatorError(Exception):
     pass
@@ -29,17 +41,24 @@ class RuleEvaluator:
     Set as a class attribute on Rule.
 
     Ensure the `model.field` is registered with `site_reference_configs`.
-    See `edc_reference`.
+
+    Note: the predicate, which is a callable, will create a Reference
+    model instance if it does not exist. See `ReferenceGetter` in
+    `edc_reference`.
+
+    See also RuleGroup and its metaclass.
     """
 
     def __init__(
-        self, logic: Optional[Any] = None, visit: Optional[Any] = None, **kwargs
+        self, logic: Logic = None, related_visit: RelatedVisitModel = None, **kwargs
     ) -> None:
-        self._registered_subject = None
-        self.logic = logic
-        self.result = None
-        self.visit = visit
-        options = dict(visit=self.visit, registered_subject=self.registered_subject, **kwargs)
+        self._registered_subject: RegisteredSubject | None = None
+        self.logic: Logic = logic
+        self.result: str | None = None
+        self.related_visit = related_visit
+        options = dict(
+            visit=self.related_visit, registered_subject=self.registered_subject, **kwargs
+        )
         try:
             predicate = self.logic.predicate(**options)
         except NoValueError as e:
@@ -68,12 +87,12 @@ class RuleEvaluator:
         if not self._registered_subject:
             try:
                 self._registered_subject = self.registered_subject_model.objects.get(
-                    subject_identifier=self.visit.subject_identifier
+                    subject_identifier=self.related_visit.subject_identifier
                 )
             except ObjectDoesNotExist as e:
                 raise RuleEvaluatorRegisterSubjectError(
                     f"Registered subject required for rule {repr(self)}. "
-                    f"subject_identifier='{self.visit.subject_identifier}'. "
+                    f"subject_identifier='{self.related_visit.subject_identifier}'. "
                     f"Got {e}."
                 )
         return self._registered_subject
