@@ -8,6 +8,14 @@ from edc_reference.site_reference import site_reference_configs
 style = color_style()
 
 
+class RuleGroupError(Exception):
+    pass
+
+
+class TargetModelConflict(Exception):
+    pass
+
+
 class RuleGroup:
 
     """Base class for CRF and Requisition rule groups."""
@@ -24,6 +32,11 @@ class RuleGroup:
         # TODO: extend this list
         default_fields = ["gender"]
 
+        if cls._meta.related_visit_model:
+            cls._lookup_model(
+                model=cls._meta.related_visit_model, category="related_visit_model"
+            )
+
         # verify models exists
         if cls._meta.source_model:
             cls._lookup_model(model=cls._meta.source_model, category="source")
