@@ -1,4 +1,6 @@
-from typing import Any
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Type
 
 from django.db import models
 
@@ -11,39 +13,48 @@ from ...metadata import (
     RequisitionMetadataGetter,
 )
 from ...metadata_rules import MetadataRuleEvaluator
-from ...stubs import SubjectVisitLikeModelObject
+
+if TYPE_CHECKING:
+    from edc_visit_schedule import Visit
+    from edc_visit_tracking.typing_stubs import RelatedVisitProtocol
+else:
+
+    class RelatedVisitProtocol:
+        ...
 
 
-class CreatesMetadataModelMixin(models.Model):
+class CreatesMetadataModelMixin(RelatedVisitProtocol, models.Model):
     """A model mixin for visit models to enable them to
     create metadata on save.
     """
 
-    metadata_cls = Metadata
-    metadata_destroyer_cls = Destroyer
-    metadata_rule_evaluator_cls = MetadataRuleEvaluator
+    metadata_cls: Type[Metadata] = Metadata
+    metadata_destroyer_cls: Type[Destroyer] = Destroyer
+    metadata_rule_evaluator_cls: Type[MetadataRuleEvaluator] = MetadataRuleEvaluator
 
     def metadata_create(self) -> None:
         """Creates metadata, called by post_save signal."""
-        metadata = self.metadata_cls(visit_model_instance=self, update_keyed=True)
+        metadata = self.metadata_cls(related_visit=self, update_keyed=True)
         metadata.prepare()
 
-    def run_metadata_rules(self) -> None:
+    def run_metadata_rules(self, allow_create: bool | None = None) -> None:
         """Runs all the metadata rules.
 
         Initially called by post_save signal.
 
         Also called by post_save signal after metadata is updated.
         """
-        metadata_rule_evaluator = self.metadata_rule_evaluator_cls(visit_model_instance=self)
+        metadata_rule_evaluator = self.metadata_rule_evaluator_cls(
+            related_visit=self, allow_create=allow_create
+        )
         metadata_rule_evaluator.evaluate_rules()
 
     @property
-    def metadata_query_options(self: SubjectVisitLikeModelObject) -> dict:
+    def metadata_query_options(self) -> dict:
         """Returns a dictionary of query options needed select
-        the visit model instance
+        the related_visit.
         """
-        visit = self.visits.get(self.appointment.visit_code)
+        visit: Visit = self.visits.get(self.appointment.visit_code)
         options = dict(
             visit_schedule_name=self.appointment.visit_schedule_name,
             schedule_name=self.appointment.schedule_name,
@@ -54,7 +65,7 @@ class CreatesMetadataModelMixin(models.Model):
         return options
 
     @property
-    def metadata(self: Any) -> Any:
+    def metadata(self) -> dict:
         """Returns a dictionary of metadata querysets for each
         metadata category (CRF or REQUISITION).
         """
@@ -65,7 +76,7 @@ class CreatesMetadataModelMixin(models.Model):
         metadata[REQUISITION] = getter.metadata_objects
         return metadata
 
-    def metadata_delete_for_visit(self: Any) -> None:
+    def metadata_delete_for_visit(self) -> None:
         """Deletes metadata for a visit when the visit is deleted.
 
         See signals.
@@ -76,7 +87,7 @@ class CreatesMetadataModelMixin(models.Model):
                     f"Metadata cannot be deleted. {key}s have been "
                     f"keyed. Got {repr(self)}."
                 )
-        destroyer = self.metadata_destroyer_cls(visit_model_instance=self)
+        destroyer = self.metadata_destroyer_cls(related_visit=self)
         destroyer.delete()
 
     class Meta:
