@@ -1,6 +1,5 @@
-from collections import OrderedDict
-
-from django.test import TestCase, override_settings
+from django.test import TestCase, override_settings, tag
+from edc_appointment.models import Appointment
 from edc_constants.constants import FEMALE, MALE
 from edc_facility.import_holidays import import_holidays
 from edc_reference.site_reference import site_reference_configs
@@ -8,6 +7,7 @@ from edc_registration.models import RegisteredSubject
 from edc_utils import get_utcnow
 from edc_visit_schedule.site_visit_schedules import site_visit_schedules
 from edc_visit_tracking.constants import SCHEDULED
+from edc_visit_tracking.models import SubjectVisit
 from faker import Faker
 
 from edc_metadata.tests.reference_configs import register_to_site_reference_configs
@@ -22,11 +22,11 @@ from ...metadata_rules import (
     PredicateError,
     RuleEvaluatorRegisterSubjectError,
     RuleGroupMetaError,
+    TargetModelConflict,
     site_metadata_rules,
 )
 from ...models import CrfMetadata
-from ...target_handler import TargetModelConflict
-from ..models import Appointment, CrfOne, SubjectConsent, SubjectVisit
+from ..models import CrfOne, SubjectConsent
 from ..visit_schedule import visit_schedule
 
 fake = Faker()
@@ -52,10 +52,10 @@ class CrfRuleGroupWithSourceModel(CrfRuleGroup):
     class Meta:
         app_label = "edc_metadata"
         source_model = "edc_metadata.crfone"
+        related_visit_model = "edc_visit_tracking.subjectvisit"
 
 
 class CrfRuleGroupWithoutSourceModel(CrfRuleGroup):
-
     crfs_male = CrfRule(
         predicate=P("gender", "eq", MALE),
         consequence=REQUIRED,
@@ -72,10 +72,10 @@ class CrfRuleGroupWithoutSourceModel(CrfRuleGroup):
 
     class Meta:
         app_label = "edc_metadata"
+        related_visit_model = "edc_visit_tracking.subjectvisit"
 
 
 class CrfRuleGroupWithoutExplicitReferenceModel(CrfRuleGroup):
-
     crfs_male = CrfRule(
         predicate=P("gender", "eq", MALE),
         consequence=REQUIRED,
@@ -93,10 +93,10 @@ class CrfRuleGroupWithoutExplicitReferenceModel(CrfRuleGroup):
     class Meta:
         app_label = "edc_metadata"
         source_model = "edc_metadata.crfone"
+        related_visit_model = "edc_visit_tracking.subjectvisit"
 
 
 class CrfRuleGroupGender(CrfRuleGroup):
-
     crfs_male = CrfRule(
         predicate=P("gender", "eq", MALE),
         consequence=REQUIRED,
@@ -113,6 +113,7 @@ class CrfRuleGroupGender(CrfRuleGroup):
 
     class Meta:
         app_label = "edc_metadata"
+        related_visit_model = "edc_visit_tracking.subjectvisit"
 
 
 class TestMetadataRulesWithGender(TestCase):
@@ -128,7 +129,7 @@ class TestMetadataRulesWithGender(TestCase):
 
         register_to_site_reference_configs()
         site_reference_configs.register_from_visit_schedule(
-            visit_models={"edc_appointment.appointment": "edc_metadata.subjectvisit"}
+            visit_models={"edc_appointment.appointment": "edc_visit_tracking.subjectvisit"}
         )
 
         # note crfs in visit schedule are all set to REQUIRED by default.
@@ -136,7 +137,7 @@ class TestMetadataRulesWithGender(TestCase):
             "edc_metadata.onschedule"
         )
 
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(rule_group_cls=CrfRuleGroupGender)
 
     def enroll(self, gender=None):
@@ -156,8 +157,13 @@ class TestMetadataRulesWithGender(TestCase):
         )
         subject_visit = SubjectVisit.objects.create(
             appointment=self.appointment,
-            reason=SCHEDULED,
             subject_identifier=subject_identifier,
+            report_datetime=self.appointment.appt_datetime,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         return subject_visit
 
@@ -214,7 +220,7 @@ class TestMetadataRulesWithGender(TestCase):
                         },
                     )
 
-    def test_rules_skipped_if_no_source_model_instance(self):
+    def test_rules_if_no_source_model_instance(self):
         subject_visit = self.enroll(MALE)
         for rule in CrfRuleGroupWithSourceModel._meta.options.get("rules"):
             with self.subTest(rule=rule):
@@ -223,16 +229,16 @@ class TestMetadataRulesWithGender(TestCase):
                     self.assertEqual(
                         result,
                         {
-                            "edc_metadata.crffive": None,
-                            "edc_metadata.crffour": None,
+                            "edc_metadata.crffive": NOT_REQUIRED,
+                            "edc_metadata.crffour": NOT_REQUIRED,
                         },
                     )
                 elif rule.name == "crfs_female":
                     self.assertEqual(
                         result,
                         {
-                            "edc_metadata.crfthree": None,
-                            "edc_metadata.crftwo": None,
+                            "edc_metadata.crfthree": NOT_REQUIRED,
+                            "edc_metadata.crftwo": NOT_REQUIRED,
                         },
                     )
 
@@ -346,7 +352,7 @@ class TestMetadataRulesWithGender(TestCase):
 
     def test_rule_group_metadata_objects(self):
         subject_visit = self.enroll(gender=MALE)
-        _, metadata_objects = CrfRuleGroupGender().evaluate_rules(visit=subject_visit)
+        _, metadata_objects = CrfRuleGroupGender().evaluate_rules(related_visit=subject_visit)
         self.assertEqual(metadata_objects.get("edc_metadata.crffour").entry_status, REQUIRED)
         self.assertEqual(metadata_objects.get("edc_metadata.crffive").entry_status, REQUIRED)
         self.assertEqual(
@@ -359,7 +365,7 @@ class TestMetadataRulesWithGender(TestCase):
 
     def test_rule_group_rule_results(self):
         subject_visit = self.enroll(gender=MALE)
-        rule_results, _ = CrfRuleGroupGender().evaluate_rules(visit=subject_visit)
+        rule_results, _ = CrfRuleGroupGender().evaluate_rules(related_visit=subject_visit)
         self.assertEqual(
             rule_results["CrfRuleGroupGender.crfs_male"].get("edc_metadata.crffour"),
             REQUIRED,
@@ -378,8 +384,7 @@ class TestMetadataRulesWithGender(TestCase):
         )
 
     def test_bad_rule_group_target_model_cannot_also_be_source_model(self):
-
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         subject_visit = self.enroll(gender=MALE)
 
         class BadCrfRuleGroup(CrfRuleGroup):
@@ -393,13 +398,15 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfone"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
         self.assertRaises(
-            CrfRuleModelConflict, BadCrfRuleGroup().evaluate_rules, visit=subject_visit
+            CrfRuleModelConflict, BadCrfRuleGroup().evaluate_rules, related_visit=subject_visit
         )
 
+    @tag("1")
     def test_rule_group_target_model_cannot_be_visit_model(self):
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
 
         subject_visit = self.enroll(gender=MALE)
 
@@ -414,9 +421,10 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfone"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
         self.assertRaises(
-            TargetModelConflict, BadCrfRuleGroup().evaluate_rules, visit=subject_visit
+            TargetModelConflict, BadCrfRuleGroup().evaluate_rules, related_visit=subject_visit
         )
 
     def test_bad_predicate_blah_is_not_an_operator(self):
@@ -433,6 +441,7 @@ class TestMetadataRulesWithGender(TestCase):
                 class Meta:
                     app_label = "edc_metadata"
                     source_model = "edc_metadata.crfone"
+                    related_visit_model = "edc_visit_tracking.subjectvisit"
 
         except PredicateError:
             pass
@@ -440,8 +449,6 @@ class TestMetadataRulesWithGender(TestCase):
             self.fail("PredicateError not raised.")
 
     def test_predicate_pf(self):
-        """Asserts entry status unchanged if source model does not exist."""
-
         def func(f1, f2):
             return True if f1 == "f1" and f2 == "f2" else False
 
@@ -456,18 +463,19 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfone"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         subject_visit = self.enroll(gender=MALE)
 
-        MyCrfRuleGroup().evaluate_rules(visit=subject_visit)
+        MyCrfRuleGroup().evaluate_rules(related_visit=subject_visit)
 
         obj = CrfMetadata.objects.get(
             model="edc_metadata.crftwo",
             subject_identifier=subject_visit.subject_identifier,
             visit_code=subject_visit.visit_code,
         )
-        self.assertEqual(obj.entry_status, REQUIRED)
+        self.assertEqual(obj.entry_status, NOT_REQUIRED)
 
     @override_settings(DEBUG=True)
     def test_predicate_pf2(self):
@@ -489,8 +497,9 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfone"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(MyCrfRuleGroup)
         subject_visit = self.enroll(gender=MALE)
         obj = CrfMetadata.objects.get(
@@ -498,10 +507,10 @@ class TestMetadataRulesWithGender(TestCase):
             subject_identifier=subject_visit.subject_identifier,
             visit_code=subject_visit.visit_code,
         )
-        self.assertEqual(obj.entry_status, REQUIRED)
+        self.assertEqual(obj.entry_status, NOT_REQUIRED)
         CrfOne.objects.create(subject_visit=subject_visit, f1="blah", f2="f2_value")
         self.assertFalse(func("blah", "f2_value"))
-        # MyCrfRuleGroup().evaluate_rules(visit=subject_visit)
+        # MyCrfRuleGroup().evaluate_rules(related_visit=subject_visit)
         obj = CrfMetadata.objects.get(
             model="edc_metadata.crftwo",
             subject_identifier=subject_visit.subject_identifier,
@@ -529,11 +538,12 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfone"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         subject_visit = self.enroll(gender=MALE)
         CrfOne.objects.create(subject_visit=subject_visit, f1="f1", f2="f2")
-        MyCrfRuleGroup().evaluate_rules(visit=subject_visit)
+        MyCrfRuleGroup().evaluate_rules(related_visit=subject_visit)
 
         obj = CrfMetadata.objects.get(
             model="edc_metadata.crftwo",
@@ -543,7 +553,6 @@ class TestMetadataRulesWithGender(TestCase):
         self.assertEqual(obj.entry_status, REQUIRED)
 
     def test_p_repr(self):
-
         p = P("blah", "eq", "car")
         self.assertTrue(repr(p))
 
@@ -575,6 +584,7 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfmissingmanager"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
         self.assertTrue(repr(MyCrfRuleGroup()._meta))
 
@@ -591,7 +601,6 @@ class TestMetadataRulesWithGender(TestCase):
                 abstract = True
 
         class NewCrfRuleGroup(CrfRuleGroup):
-
             rule2 = CrfRule(
                 predicate=P("f1", "eq", "car"),
                 consequence=REQUIRED,
@@ -602,11 +611,11 @@ class TestMetadataRulesWithGender(TestCase):
             class Meta:
                 app_label = "edc_metadata"
                 source_model = "edc_metadata.crfmissingmanager"
+                related_visit_model = "edc_visit_tracking.subjectvisit"
 
         self.assertTrue(len(NewCrfRuleGroup()._meta.options.get("rules")), 2)
 
     def test_rule_group_missing_meta(self):
-
         try:
 
             class MyCrfRuleGroup(CrfRuleGroup):
@@ -623,7 +632,6 @@ class TestMetadataRulesWithGender(TestCase):
             self.fail("AttributeError not raised.")
 
     def test_rule_group_invalid_meta_option(self):
-
         try:
 
             class MyCrfRuleGroup(CrfRuleGroup):
@@ -637,6 +645,7 @@ class TestMetadataRulesWithGender(TestCase):
                 class Meta:
                     app_label = "edc_metadata"
                     source_model = "edc_metadata.crfmissingmanager"
+                    related_visit_model = "edc_visit_tracking.subjectvisit"
                     blah = "blah"
 
         except RuleGroupMetaError:
