@@ -1,20 +1,21 @@
-from collections import OrderedDict
-
 from django.apps import apps as django_apps
+from django.core.exceptions import ObjectDoesNotExist
 from django.test import TestCase
+from edc_appointment.models import Appointment
 from edc_constants.constants import MALE
 from edc_facility.import_holidays import import_holidays
 from edc_reference.site_reference import site_reference_configs
 from edc_utils import get_utcnow
 from edc_visit_schedule.site_visit_schedules import site_visit_schedules
 from edc_visit_tracking.constants import SCHEDULED
+from edc_visit_tracking.models import SubjectVisit
 from faker import Faker
 
 from edc_metadata import KEYED, NOT_REQUIRED, REQUIRED
 from edc_metadata.models import CrfMetadata
 
 from ...metadata_rules import CrfRule, CrfRuleGroup, P, site_metadata_rules
-from ..models import Appointment, CrfOne, CrfTwo, SubjectConsent, SubjectVisit
+from ..models import CrfOne, CrfTwo, SubjectConsent
 from ..reference_configs import register_to_site_reference_configs
 from ..visit_schedule import visit_schedule
 
@@ -23,7 +24,6 @@ edc_registration_app_config = django_apps.get_app_config("edc_registration")
 
 
 class CrfRuleGroupOne(CrfRuleGroup):
-
     crfs_car = CrfRule(
         predicate=P("f1", "eq", "car"),
         consequence=REQUIRED,
@@ -44,7 +44,6 @@ class CrfRuleGroupOne(CrfRuleGroup):
 
 
 class CrfRuleGroupTwo(CrfRuleGroup):
-
     crfs_truck = CrfRule(
         predicate=P("f1", "eq", "truck"),
         consequence=REQUIRED,
@@ -65,7 +64,6 @@ class CrfRuleGroupTwo(CrfRuleGroup):
 
 
 class CrfRuleGroupThree(CrfRuleGroup):
-
     crfs_truck = CrfRule(
         predicate=P("f1", "eq", "holden"),
         consequence=REQUIRED,
@@ -90,7 +88,7 @@ class CrfRuleGroupTestCase(TestCase):
 
         register_to_site_reference_configs()
         site_reference_configs.register_from_visit_schedule(
-            visit_models={"edc_appointment.appointment": "edc_metadata.subjectvisit"}
+            visit_models={"edc_appointment.appointment": "edc_visit_tracking.subjectvisit"}
         )
 
         # note crfs in visit schedule are all set to REQUIRED by default.
@@ -98,7 +96,7 @@ class CrfRuleGroupTestCase(TestCase):
             "edc_metadata.onschedule"
         )
 
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(rule_group_cls=CrfRuleGroupOne)
         site_metadata_rules.register(rule_group_cls=CrfRuleGroupTwo)
         site_metadata_rules.register(rule_group_cls=CrfRuleGroupThree)
@@ -120,12 +118,13 @@ class CrfRuleGroupTestCase(TestCase):
         )
         subject_visit = SubjectVisit.objects.create(
             appointment=self.appointment,
-            reason=SCHEDULED,
             subject_identifier=subject_identifier,
+            report_datetime=self.appointment.appt_datetime,
             visit_code=self.appointment.visit_code,
             visit_code_sequence=self.appointment.visit_code_sequence,
             visit_schedule_name=self.appointment.visit_schedule_name,
             schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         return subject_visit
 
@@ -140,16 +139,43 @@ class CrfRuleGroupTestCase(TestCase):
             visit_code_sequence=self.appointment.next.visit_code_sequence,
         )
 
+    def test_default_d1(self):
+        """Test before any CRFs are submitted"""
+        self.enroll(gender=MALE)
+        self.assertEqual(
+            CrfMetadata.objects.get(model="edc_metadata.crfone").entry_status,
+            REQUIRED,
+        )
+        # set to NOT_REQUIRED by CrfRuleGroupOne.crfs_car
+        self.assertEqual(
+            CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
+            NOT_REQUIRED,
+        )
+        # set to NOT_REQUIRED by CrfRuleGroupOne.crfs_bicycle
+        self.assertEqual(
+            CrfMetadata.objects.get(model="edc_metadata.crfthree").entry_status,
+            NOT_REQUIRED,
+        )
+        self.assertEqual(
+            CrfMetadata.objects.get(model="edc_metadata.crffour").entry_status,
+            REQUIRED,
+        )
+        # set to NOT_REQUIRED by CrfRuleGroupTwo.crfs_truck
+        self.assertEqual(
+            CrfMetadata.objects.get(model="edc_metadata.crffive").entry_status,
+            NOT_REQUIRED,
+        )
+
     def test_example1(self):
         """Asserts CrfTwo is REQUIRED if f1==\'car\' as specified."""
         subject_visit = self.enroll(gender=MALE)
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crfthree").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
         CrfOne.objects.create(subject_visit=subject_visit, f1="car")
@@ -169,11 +195,11 @@ class CrfRuleGroupTestCase(TestCase):
         subject_visit = self.enroll(gender=MALE)
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crfthree").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
         CrfOne.objects.create(subject_visit=subject_visit, f3="bicycle")
@@ -206,11 +232,11 @@ class CrfRuleGroupTestCase(TestCase):
         subject_visit = self.enroll(gender=MALE)
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crfthree").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
         crf_one = CrfOne.objects.create(subject_visit=subject_visit, f1="not car")
@@ -253,7 +279,7 @@ class CrfRuleGroupTestCase(TestCase):
         subject_visit = self.enroll(gender=MALE)
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
         CrfTwo.objects.create(subject_visit=subject_visit)
@@ -281,9 +307,13 @@ class CrfRuleGroupTestCase(TestCase):
     def test_recovers_from_missing_metadata(self):
         subject_visit = self.enroll(gender=MALE)
         metadata_obj = CrfMetadata.objects.get(model="edc_metadata.crftwo")
-        self.assertEqual(metadata_obj.entry_status, REQUIRED)
+        self.assertEqual(metadata_obj.entry_status, NOT_REQUIRED)
 
+        # note, does not automatically recreate
         metadata_obj.delete()
+        self.assertRaises(
+            ObjectDoesNotExist, CrfMetadata.objects.get, model="edc_metadata.crftwo"
+        )
 
         CrfTwo.objects.create(subject_visit=subject_visit)
 
@@ -295,7 +325,7 @@ class CrfRuleGroupTestCase(TestCase):
         subject_visit = self.enroll(gender=MALE)
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
         crf_two = CrfTwo.objects.create(subject_visit=subject_visit)
@@ -309,7 +339,7 @@ class CrfRuleGroupTestCase(TestCase):
 
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
     def test_delete_2(self):
@@ -317,7 +347,7 @@ class CrfRuleGroupTestCase(TestCase):
         subject_visit = self.enroll(gender=MALE)
         self.assertEqual(
             CrfMetadata.objects.get(model="edc_metadata.crftwo").entry_status,
-            REQUIRED,
+            NOT_REQUIRED,
         )
 
         crf_two = CrfTwo.objects.create(subject_visit=subject_visit)
@@ -363,14 +393,17 @@ class CrfRuleGroupTestCase(TestCase):
             ).entry_status,
             REQUIRED,
         )
-        self.assertEqual(
+        try:
             CrfMetadata.objects.get(
                 model="edc_metadata.prnone",
                 visit_code=subject_visit_two.visit_code,
                 visit_code_sequence=subject_visit_two.visit_code_sequence,
-            ).entry_status,
-            NOT_REQUIRED,
-        )
+            )
+        except ObjectDoesNotExist:
+            pass
+        else:
+            self.fail("CrfMetadata model instance unexpectedly exists")
+
         crf_one.f1 = "caufield"
         crf_one.save()
         self.assertEqual(
@@ -381,14 +414,16 @@ class CrfRuleGroupTestCase(TestCase):
             ).entry_status,
             NOT_REQUIRED,
         )
-        self.assertEqual(
+        try:
             CrfMetadata.objects.get(
                 model="edc_metadata.prnone",
                 visit_code=subject_visit_two.visit_code,
                 visit_code_sequence=subject_visit_two.visit_code_sequence,
-            ).entry_status,
-            NOT_REQUIRED,
-        )
+            )
+        except ObjectDoesNotExist:
+            pass
+        else:
+            self.fail("CrfMetadata model instance unexpectedly exists")
         crf_one.f1 = "holden"
         crf_one.save()
         self.assertEqual(
@@ -399,14 +434,16 @@ class CrfRuleGroupTestCase(TestCase):
             ).entry_status,
             REQUIRED,
         )
-        self.assertEqual(
+        try:
             CrfMetadata.objects.get(
                 model="edc_metadata.prnone",
                 visit_code=subject_visit_two.visit_code,
                 visit_code_sequence=subject_visit_two.visit_code_sequence,
-            ).entry_status,
-            NOT_REQUIRED,
-        )
+            )
+        except ObjectDoesNotExist:
+            pass
+        else:
+            self.fail("CrfMetadata model instance unexpectedly exists")
 
     def test_prn_rule_acts_on_correct_visit2(self):
         """Asserts handles PRNs correctly"""
@@ -423,9 +460,9 @@ class CrfRuleGroupTestCase(TestCase):
 
         self.get_next_subject_visit(subject_visit)
 
-        self.assertEqual(2, CrfMetadata.objects.filter(model="edc_metadata.prnone").count())
+        self.assertEqual(1, CrfMetadata.objects.filter(model="edc_metadata.prnone").count())
         self.assertEqual(
-            2,
+            1,
             CrfMetadata.objects.filter(
                 model="edc_metadata.prnone", entry_status=NOT_REQUIRED
             ).count(),
@@ -446,7 +483,7 @@ class CrfRuleGroupTestCase(TestCase):
             ).count(),
         )
         self.assertEqual(
-            1,
+            0,
             CrfMetadata.objects.filter(
                 model="edc_metadata.prnone", entry_status=NOT_REQUIRED
             ).count(),
