@@ -1,10 +1,9 @@
-from django.test import TestCase, tag
+from django.test import TestCase
 
 from ...constants import NOT_REQUIRED, REQUIRED
 from ...metadata_rules import Logic, RuleLogicError
 
 
-@tag("12")
 class MetadataRulesTests(TestCase):
     def test_logic(self):
         logic = Logic(
