@@ -36,7 +36,12 @@ class TestMetadataRefresher(TestMetadataMixin, TestCase):
 
     def test_updates_crf_metadata_as_keyed(self):
         subject_visit = SubjectVisit.objects.create(
-            appointment=self.appointment, reason=SCHEDULED
+            appointment=self.appointment,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         CrfOne.objects.create(subject_visit=subject_visit)
         CrfMetadata.objects.all().delete()
@@ -51,7 +56,12 @@ class TestMetadataRefresher(TestMetadataMixin, TestCase):
 
     def test_updates_after_manual_change(self):
         subject_visit = SubjectVisit.objects.create(
-            appointment=self.appointment, reason=SCHEDULED
+            appointment=self.appointment,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         CrfOne.objects.create(subject_visit=subject_visit)
         CrfMetadata.objects.all().delete()
@@ -71,7 +81,12 @@ class TestMetadataRefresher(TestMetadataMixin, TestCase):
 
     def test_updates_after_manual_change2(self):
         subject_visit = SubjectVisit.objects.create(
-            appointment=self.appointment, reason=SCHEDULED
+            appointment=self.appointment,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         crf_one = CrfOne.objects.create(subject_visit=subject_visit)
         CrfMetadata.objects.all().delete()
@@ -116,7 +131,7 @@ class TestMetadataRefresher(TestMetadataMixin, TestCase):
         schedule.add_visit(visit)
         new_visit_schedule = VisitSchedule(
             name="visit_schedule",
-            offstudy_model="edc_metadata.subjectoffstudy",
+            offstudy_model="edc_offstudy.subjectoffstudy",
             death_report_model="edc_metadata.deathreport",
         )
         new_visit_schedule.add_schedule(schedule)
@@ -124,7 +139,12 @@ class TestMetadataRefresher(TestMetadataMixin, TestCase):
 
     def test_after_schedule_change(self):
         subject_visit = SubjectVisit.objects.create(
-            appointment=self.appointment, reason=SCHEDULED
+            appointment=self.appointment,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         CrfFive.objects.create(subject_visit=subject_visit)
         self.assertEqual(CrfMetadata.objects.all().count(), 5)
@@ -138,19 +158,27 @@ class TestMetadataRefresher(TestMetadataMixin, TestCase):
         self.assertEqual(CrfMetadata.objects.all().count(), 5)
         metadata_refresher = MetadataRefresher()
         metadata_refresher.run()
-        self.assertEqual(CrfMetadata.objects.all().count(), 5)
+        # CrfFive is no longer in visit schedule, so one less
+        # CRFMetadata instance
+        self.assertEqual(CrfMetadata.objects.all().count(), 4)
         expected = {
             "edc_metadata.crfone": REQUIRED,
             "edc_metadata.crftwo": REQUIRED,
             "edc_metadata.crfthree": REQUIRED,
             "edc_metadata.crfFour": REQUIRED,
-            "edc_metadata.crfFive": KEYED,
         }
         self.check(expected, subject_visit=subject_visit)
 
     def test_after_schedule_change2(self):
         subject_visit = SubjectVisit.objects.create(
-            appointment=self.appointment, reason=SCHEDULED
+            appointment=self.appointment,
+            subject_identifier=self.subject_identifier,
+            report_datetime=self.appointment.appt_datetime,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         self.assertEqual(CrfMetadata.objects.all().count(), 5)
         crfs = FormsCollection(
