@@ -1,12 +1,12 @@
 from django.core.exceptions import ObjectDoesNotExist
-from django.test import TestCase
+from django.test import TestCase, tag
 from edc_visit_tracking.constants import SCHEDULED
 
 from ...constants import KEYED, NOT_REQUIRED, REQUIRED
+from ...metadata_handler import MetadataHandlerError
 from ...metadata_inspector import MetaDataInspector
 from ...metadata_updater import MetadataUpdater
 from ...models import CrfMetadata, RequisitionMetadata
-from ...target_handler import TargetModelLookupError, TargetModelNotScheduledForVisit
 from ..models import CrfOne, CrfThree, CrfTwo, SubjectRequisition, SubjectVisit
 from .metadata_test_mixin import TestMetadataMixin
 
@@ -14,7 +14,14 @@ from .metadata_test_mixin import TestMetadataMixin
 class TestMetadataUpdater(TestMetadataMixin, TestCase):
     def test_updates_crf_metadata_as_keyed(self):
         subject_visit = SubjectVisit.objects.create(
-            appointment=self.appointment, reason=SCHEDULED
+            appointment=self.appointment,
+            subject_identifier=self.subject_identifier,
+            report_datetime=self.appointment.appt_datetime,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         CrfOne.objects.create(subject_visit=subject_visit)
         self.assertEqual(
@@ -141,6 +148,7 @@ class TestMetadataUpdater(TestMetadataMixin, TestCase):
             1,
         )
 
+    @tag("2")
     def test_resets_requisition_metadata_on_delete2(self):
         subject_visit = SubjectVisit.objects.create(
             appointment=self.appointment, reason=SCHEDULED
@@ -232,10 +240,10 @@ class TestMetadataUpdater(TestMetadataMixin, TestCase):
             entry_status=REQUIRED,
         )
         metadata_updater = MetadataUpdater(
-            visit_model_instance=subject_visit,
+            related_visit=subject_visit,
             target_model="edc_metadata.crfone",
         )
-        metadata_updater.update(entry_status=NOT_REQUIRED)
+        metadata_updater.get_and_update(entry_status=NOT_REQUIRED)
         self.assertRaises(
             ObjectDoesNotExist,
             CrfMetadata.objects.get,
@@ -268,11 +276,11 @@ class TestMetadataUpdater(TestMetadataMixin, TestCase):
             appointment=self.appointment, reason=SCHEDULED
         )
         metadata_updater = MetadataUpdater(
-            visit_model_instance=subject_visit,
+            related_visit=subject_visit,
             target_model="edc_metadata.blah",
         )
         self.assertRaises(
-            TargetModelLookupError, metadata_updater.update, entry_status=NOT_REQUIRED
+            MetadataHandlerError, metadata_updater.get_and_update, entry_status=NOT_REQUIRED
         )
 
     def test_crf_model_not_scheduled(self):
@@ -280,11 +288,11 @@ class TestMetadataUpdater(TestMetadataMixin, TestCase):
             appointment=self.appointment, reason=SCHEDULED
         )
         metadata_updater = MetadataUpdater(
-            visit_model_instance=subject_visit,
+            related_visit=subject_visit,
             target_model="edc_metadata.crfseven",
         )
         self.assertRaises(
-            TargetModelNotScheduledForVisit,
-            metadata_updater.update,
+            MetadataHandlerError,
+            metadata_updater.get_and_update,
             entry_status=NOT_REQUIRED,
         )
