@@ -1,6 +1,6 @@
-from collections import OrderedDict
-
+from django.core.exceptions import ObjectDoesNotExist
 from django.test import TestCase
+from edc_appointment.models import Appointment
 from edc_constants.constants import FEMALE, MALE
 from edc_facility.import_holidays import import_holidays
 from edc_lab.models import Panel
@@ -9,6 +9,7 @@ from edc_reference.models import Reference
 from edc_utils import get_utcnow
 from edc_visit_schedule.site_visit_schedules import site_visit_schedules
 from edc_visit_tracking.constants import SCHEDULED
+from edc_visit_tracking.models import SubjectVisit
 from faker import Faker
 
 from ...constants import KEYED, NOT_REQUIRED, REQUIRED
@@ -20,13 +21,7 @@ from ...metadata_rules import (
     site_metadata_rules,
 )
 from ...models import RequisitionMetadata
-from ..models import (
-    Appointment,
-    CrfOne,
-    SubjectConsent,
-    SubjectRequisition,
-    SubjectVisit,
-)
+from ..models import CrfOne, SubjectConsent, SubjectRequisition
 from ..reference_configs import register_to_site_reference_configs
 from ..visit_schedule import visit_schedule
 
@@ -110,7 +105,6 @@ class RequisitionRuleGroup3(RequisitionRuleGroup):
 
 
 class BaseRequisitionRuleGroup(RequisitionRuleGroup):
-
     male = RequisitionRule(
         predicate=P("gender", "eq", MALE),
         consequence=REQUIRED,
@@ -160,12 +154,12 @@ class TestRequisitionRuleGroup(TestCase):
 
         register_to_site_reference_configs()
         site_reference_configs.register_from_visit_schedule(
-            visit_models={"edc_appointment.appointment": "edc_metadata.subjectvisit"}
+            visit_models={"edc_appointment.appointment": "edc_visit_tracking.subjectvisit"}
         )
         _, self.schedule = site_visit_schedules.get_by_onschedule_model(
             "edc_metadata.onschedule"
         )
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
 
     def enroll(self, gender=None):
         """Returns a subject visit model after enrolling a
@@ -187,14 +181,19 @@ class TestRequisitionRuleGroup(TestCase):
         )
         subject_visit = SubjectVisit.objects.create(
             appointment=self.appointment,
-            reason=SCHEDULED,
             subject_identifier=subject_identifier,
+            report_datetime=self.appointment.appt_datetime,
+            visit_code=self.appointment.visit_code,
+            visit_code_sequence=self.appointment.visit_code_sequence,
+            visit_schedule_name=self.appointment.visit_schedule_name,
+            schedule_name=self.appointment.schedule_name,
+            reason=SCHEDULED,
         )
         return subject_visit
 
     def test_rule_male(self):
         subject_visit = self.enroll(gender=MALE)
-        rule_results, _ = MyRequisitionRuleGroup().evaluate_rules(visit=subject_visit)
+        rule_results, _ = MyRequisitionRuleGroup().evaluate_rules(related_visit=subject_visit)
         for panel in [self.panel_one, self.panel_two]:
             with self.subTest(panel=panel):
                 key = "edc_metadata.subjectrequisition"
@@ -205,7 +204,7 @@ class TestRequisitionRuleGroup(TestCase):
 
     def test_rule_female(self):
         subject_visit = self.enroll(gender=FEMALE)
-        rule_results, _ = MyRequisitionRuleGroup().evaluate_rules(visit=subject_visit)
+        rule_results, _ = MyRequisitionRuleGroup().evaluate_rules(related_visit=subject_visit)
         for panel in [self.panel_one, self.panel_two]:
             with self.subTest(panel=panel):
                 key = "edc_metadata.subjectrequisition"
@@ -215,7 +214,6 @@ class TestRequisitionRuleGroup(TestCase):
                     self.assertEqual(rule_result.entry_status, NOT_REQUIRED)
 
     def test_source_panel_required_raises(self):
-
         try:
 
             class BadRequisitionRuleGroup(BaseRequisitionRuleGroup):
@@ -236,7 +234,6 @@ class TestRequisitionRuleGroup(TestCase):
         try:
 
             class BadRequisitionRuleGroup(RequisitionRuleGroup):
-
                 male = RequisitionRule(
                     predicate=P("gender", "eq", MALE),
                     consequence=REQUIRED,
@@ -268,7 +265,7 @@ class TestRequisitionRuleGroup(TestCase):
 
     def test_rule_male_with_source_model_as_requisition(self):
         subject_visit = self.enroll(gender=MALE)
-        rule_results, _ = RequisitionRuleGroup2().evaluate_rules(visit=subject_visit)
+        rule_results, _ = RequisitionRuleGroup2().evaluate_rules(related_visit=subject_visit)
         for panel_name in ["one", "two"]:
             with self.subTest(panel_name=panel_name):
                 key = "edc_metadata.subjectrequisition"
@@ -279,7 +276,7 @@ class TestRequisitionRuleGroup(TestCase):
 
     def test_metadata_for_rule_male_with_source_model_as_requisition1(self):
         """RequisitionRuleGroup2"""
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup2)
         subject_visit = self.enroll(gender=MALE)
         SubjectRequisition.objects.create(subject_visit=subject_visit, panel=self.panel_five)
@@ -295,7 +292,7 @@ class TestRequisitionRuleGroup(TestCase):
 
     def test_metadata_for_rule_male_with_source_model_as_requisition2(self):
         subject_visit = self.enroll(gender=MALE)
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup2)
         SubjectRequisition.objects.create(subject_visit=subject_visit, panel=self.panel_five)
         for panel_name in ["three", "four"]:
@@ -310,7 +307,7 @@ class TestRequisitionRuleGroup(TestCase):
 
     def test_metadata_for_rule_female_with_source_model_as_requisition1(self):
         subject_visit = self.enroll(gender=FEMALE)
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup2)
         Reference.objects.create(
             visit_schedule_name=subject_visit.visit_schedule_name,
@@ -335,7 +332,7 @@ class TestRequisitionRuleGroup(TestCase):
 
     def test_metadata_for_rule_female_with_source_model_as_requisition2(self):
         subject_visit = self.enroll(gender=FEMALE)
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup2)
         SubjectRequisition.objects.create(subject_visit=subject_visit, panel=self.panel_five)
         for panel in [self.panel_one, self.panel_two]:
@@ -349,7 +346,7 @@ class TestRequisitionRuleGroup(TestCase):
                 self.assertEqual(obj.entry_status, NOT_REQUIRED)
 
     def test_metadata_requisition(self):
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup3)
 
         subject_visit = self.enroll(gender=FEMALE)
@@ -397,7 +394,7 @@ class TestRequisitionRuleGroup(TestCase):
     # TODO: fix
     def test_keyed_instance_ignores_rules(self):
         """Asserts if instance exists, rule is ignored"""
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup3)
 
         subject_visit = self.enroll(gender=FEMALE)
@@ -480,7 +477,7 @@ class TestRequisitionRuleGroup(TestCase):
     def test_recovers_from_sequence_problem(self):
         """Asserts if instance exists, rule is ignored."""
         subject_visit = self.enroll(gender=FEMALE)
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup3)
         Reference.objects.create(
             visit_schedule_name=subject_visit.visit_schedule_name,
@@ -530,21 +527,21 @@ class TestRequisitionRuleGroup(TestCase):
         self.assertEqual(metadata_obj.entry_status, KEYED)
 
     def test_recovers_from_missing_metadata(self):
-        subject_visit = self.enroll(gender=FEMALE)
-        site_metadata_rules.registry = OrderedDict()
+        site_metadata_rules.registry = {}
         site_metadata_rules.register(RequisitionRuleGroup3)
-        Reference.objects.create(
-            visit_schedule_name=subject_visit.visit_schedule_name,
-            schedule_name=subject_visit.schedule_name,
-            visit_code=subject_visit.visit_code,
-            timepoint=subject_visit.timepoint,
-            identifier=subject_visit.subject_identifier,
-            report_datetime=subject_visit.report_datetime,
-            field_name="panel",
-            value_uuid=self.panel_five.id,
-        )
+        subject_visit = self.enroll(gender=FEMALE)
+        try:
+            RequisitionMetadata.objects.get(
+                model="edc_metadata.subjectrequisition",
+                subject_identifier=subject_visit.subject_identifier,
+                visit_code=subject_visit.visit_code,
+                panel_name=self.panel_six.name,
+                entry_status=NOT_REQUIRED,
+            )
+        except ObjectDoesNotExist:
+            self.fail("RequisitionMetadata unexepectedly does not exist")
 
-        # create CRF that triggers rule to REQUIRED
+        # create CRF that triggers rule to panel_six = REQUIRED
         crf_one = CrfOne.objects.create(subject_visit=subject_visit, f1="hello")
         metadata_obj = RequisitionMetadata.objects.get(
             model="edc_metadata.subjectrequisition",
